from rdflib import Namespace

SYNTHEA_IRI = "https://knacc.umbc.edu/dae-young/kim/ontologies/synthea#"
DUA_IRI = "https://knacc.umbc.edu/dae-young/kim/ontologies/data-usage-agreement#"
TRUST_IRI = "https://knacc.umbc.edu/dae-young/kim/ontologies/trust#"
DATA_EXCHANGE_IRI = "https://knacc.umbc.edu/dae-young/kim/ontologies/data-exchange#"

TST = Namespace(TRUST_IRI)
DUA = Namespace(DUA_IRI)
SYN = Namespace(SYNTHEA_IRI)
DEX = Namespace(DATA_EXCHANGE_IRI)
